# This code has been developed under THALIS: MATENVMED project for Action 2.1 and co-financed by Greece and EU - NSRF 2007-2013.
# This FEniCS python code is used for solving the brain problem using the Backward-Euler method and is compatible for FEniCS software ver. 1.6.0 +.

from dolfin import *
import numpy as np

# Mesh definition from GMSH files
mesh = Mesh("m05.xml")
subdomains = MeshFunction('size_t', mesh, "m05_physical_region.xml")
boundaries = MeshFunction('size_t', mesh, "m05_facet_region.xml")
parameters["reorder_dofs_serial"] = False

#plot(boundaries,interactive=True)
#plot(subdomains,interactive=True)

V0 = FunctionSpace(mesh, "DG", 0)
V1 = FunctionSpace(mesh, "CG", 1)

coor = mesh.coordinates()

# K definition for each subdomain
kappa = Function(V0)
k_values = [ 0.2, 1.0, 1.0]
help = np.asarray(subdomains.array(), dtype=np.int32)
kappa.vector()[:] = np.choose(help, k_values)

#Change initial coordinates using coor[k][0] and coor[k][1] and tumor growth factor by alpha
u0 = Expression("(1.0/(alpha*sqrt(pi)))*exp(-(pow(x[0] - 4.733, 2) + pow(x[1] - 7.877, 2)) / (alpha*alpha))", alpha = 0.1)

u_init = interpolate(u0, V1)
u_init_array = u_init.vector().array()
# Time discretization step
dt = 0.012

# Define variational problem
u = TrialFunction(V1)
v = TestFunction(V1)
f = Constant(0.0)

a = u*v*dx + dt*kappa*inner(nabla_grad(u), nabla_grad(v))*dx
L = (u_init + dt*f)*v*dx

A = assemble(a)

u = Function(V1)
T = 6
t = dt
u_prev = u_init
# Time iteration for BackwardEuler
while t <= T:
    b = assemble(L)
    solve(A, u.vector(), b)
    plot(u)
    t = t + dt
    u_prev.assign(u)
    L = (u_prev + dt*f)*v*dx
# Plot the solution
plot(u, interactive=True)



